@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2009, 2011  Tom Cato Amundsen
+# Copyright (C) 2009, 2011, 2016  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,14 +16,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from __future__ import absolute_import
-
 import logging
 import os
-import StringIO
+import io
 import subprocess
 
 from gi.repository import Gtk
+from gi.repository import Gdk
 
 from solfege.esel import SearchView
 
@@ -41,7 +40,9 @@ from solfege import frontpage as pd
 from solfege import lessonfile
 from solfege import osutils
 
+
 class LessonFilePreviewWidget(Gtk.VBox):
+
     def __init__(self, model):
         Gtk.VBox.__init__(self)
         self.m_model = model
@@ -67,10 +68,10 @@ class LessonFilePreviewWidget(Gtk.VBox):
         self.g_topic_box = Gtk.VBox()
         self.pack_start(self.g_topic_box, False, False, 0)
         self.show_all()
+
     def update(self, dlg):
         fn = dlg.get_preview_filename()
         if fn:
-            fn = gu.decode_filename(fn)
             for child in self.g_topic_box.get_children():
                 child.destroy()
             fn = lessonfile.mk_uri(fn)
@@ -84,19 +85,21 @@ class LessonFilePreviewWidget(Gtk.VBox):
                     l.set_alignment(0.0, 0.5)
                     self.g_topic_box.pack_start(l, False, False, 0)
                 if not self.g_topic_box.get_children():
-                    l = Gtk.Label(label=u"-")
+                    l = Gtk.Label(label="-")
                     l.set_alignment(0.0, 0.5)
                     self.g_topic_box.pack_start(l, False, False, 0)
             except (lessonfile.InfoCache.FileNotFound,
-                    lessonfile.InfoCache.FileNotLessonfile), e:
-                self.g_title.set_text(u'')
-                self.g_module.set_text(u'')
+                    lessonfile.InfoCache.FileNotLessonfile):
+                self.g_title.set_text('')
+                self.g_module.set_text('')
                 self.g_ok_button.set_sensitive(False)
                 self.set_sensitive(False)
         self.show_all()
         return True
 
+
 class SelectLessonFileDialog(Gtk.FileChooserDialog):
+
     def __init__(self, parent):
         Gtk.FileChooserDialog.__init__(self, _("Select lesson file"),
             parent=parent,
@@ -111,13 +114,15 @@ class SelectLessonFileDialog(Gtk.FileChooserDialog):
 
 
 class SelectLessonfileBySearchDialog(Gtk.Dialog):
-    def __init__(self):
-        Gtk.Dialog.__init__(self, buttons=(Gtk.STOCK_CLOSE, Gtk.ResponseType.ACCEPT))
+
+    def __init__(self, w):
+        Gtk.Dialog.__init__(self, buttons=(Gtk.STOCK_CLOSE, Gtk.ResponseType.ACCEPT), parent=w)
         view = SearchView(_('Search for exercises. Each exercise you click will be added to the section of the front page.'),
             fields=['link-with-filename-tooltip', 'module'])
         view.on_link_clicked = self.on_link_clicked
         self.vbox.pack_start(view, True, True, 0)
         self.show_all()
+
     def on_link_clicked(self, widget, filename):
         self.m_filename = filename
         self.response(Gtk.ResponseType.OK)
@@ -132,6 +137,7 @@ def editor_of(obj):
         p = p.m_parent
     return p
 
+
 def parent_page(obj):
     """
     Return the parent page of obj. Return None if this is the toplevel page.
@@ -147,23 +153,56 @@ def parent_page(obj):
         if p is None:
             return None
 
+
+class MyEntry(Gtk.Entry):
+
+    def __init__(self, text, finish_cb):
+        """
+        text - the default text for the entry
+        finish_cb - the function to call with text as its argument
+                    when the user is finished editing.
+        """
+        Gtk.Entry.__init__(self)
+        self.finish_edit = finish_cb
+        self.set_text(text)
+        self.m_text = text
+        self.activate_id = self.connect('activate', self.activate_cb)
+        self.focus_id = self.connect('focus-out-event', self.focus_cb)
+        self.escape_id = self.connect('key-release-event', self.escape_cb)
+
+    def activate_cb(self, entry):
+        entry.disconnect(self.focus_id)
+        entry.disconnect(self.escape_id)
+        self.finish_edit(entry.get_text())
+
+    def focus_cb(self, entry, event):
+        entry.disconnect(self.activate_id)
+        entry.disconnect(self.escape_id)
+        self.finish_edit(entry.get_text())
+        return True
+
+    def escape_cb(self, entry, event):
+        if event.keyval == Gdk.KEY_Escape:
+            entry.disconnect(self.activate_id)
+            entry.disconnect(self.focus_id)
+            self.finish_edit(self.m_text)
+            return True
+
+
 class Section(Gtk.VBox):
     """
     A section consists of a heading and a list of links.
     self.g_link_box is a vbox that contains the links.
     """
+
     def __init__(self, model, parent):
         Gtk.VBox.__init__(self)
         self.m_model = model
         self.m_parent = parent
         assert isinstance(model, pd.LinkList)
-        hbox = Gtk.HBox()
+        self.heading_hbox = hbox = Gtk.HBox()
         hbox.set_spacing(6)
         self.pack_start(hbox, False, False, 0)
-        # This is displayed and used when we edit the heading
-        self.g_heading_entry = Gtk.Entry()
-        self.g_heading_entry.set_no_show_all(True)
-        hbox.pack_start(self.g_heading_entry, True, True, 0)
         self.g_heading = Gtk.Label()
         self.g_heading.set_alignment(0.0, 0.5)
         # FIXME escape m_name
@@ -206,7 +245,7 @@ class Section(Gtk.VBox):
         b = Gtk.Button()
         b.add(im)
         b.connect('clicked', self.on_paste, -1)
-        Editor.clipboard.register_paste_button(b, (pd.LinkList, pd.Page, unicode))
+        Editor.clipboard.register_paste_button(b, (pd.LinkList, pd.Page, str))
         button_hbox.pack_start(b, False, False, 0)
         #
         im = Gtk.Image()
@@ -248,34 +287,10 @@ class Section(Gtk.VBox):
         # The button to click to add a new link
         hbox = Gtk.HBox()
         self.pack_start(hbox, True, True, 0)
-    def on_edit_heading(self, btn):
-        self.g_heading_entry.set_text(self.m_model.m_name)
-        self.g_heading_entry.show()
-        self.g_heading.hide()
-        self.g_heading_entry.grab_focus()
-        def finish_edit(entry):
-            self.g_heading_entry.disconnect(sid)
-            self.g_heading_entry.disconnect(keyup_id)
-            self.g_heading_entry.disconnect(keydown_sid)
-            self.m_model.m_name = entry.get_text()
-            self.g_heading.set_markup(u"<b>%s</b>" % entry.get_text())
-            self.g_heading_entry.hide()
-            self.g_heading.show()
-        sid = self.g_heading_entry.connect('activate', finish_edit)
-        def keydown(entry, event):
-            if event.keyval == Gdk.KEY_Tab:
-                finish_edit(entry)
-        keydown_sid = self.g_heading_entry.connect('key-press-event', keydown)
-        def keyup(entry, event):
-            if event.keyval == Gdk.KEY_Escape:
-                self.g_heading_entry.disconnect(sid)
-                self.g_heading_entry.disconnect(keyup_id)
-                self.g_heading_entry.hide()
-                self.g_heading.show()
-                return True
-        keyup_id = self.g_heading_entry.connect('key-release-event', keyup)
-    def on_add(self, btn, event):
-        menu = Gtk.Menu()
+
+        # The popup menu that will popup from the STOCK_ADD button
+        self.g_add_popup = menu = Gtk.Menu()
+        menu.show()
         item = Gtk.MenuItem(_("Add link to new page"))
         item.connect('activate', self.on_add_link_to_new_page)
         menu.append(item)
@@ -286,11 +301,56 @@ class Section(Gtk.VBox):
         item.connect('activate', self.on_add_link_by_search)
         menu.append(item)
         menu.show_all()
-        menu.popup(None, None, None, None, event.button, event.time)
+
+        # The pop menu shown when right clicking on a link to an
+        # exercise or a subpage.
+        self.popup_menu = m = Gtk.Menu()
+        item = Gtk.ImageMenuItem(Gtk.STOCK_DELETE)
+        item.connect('activate', self.on_delete_link)
+        m.append(item)
+        item = Gtk.ImageMenuItem(Gtk.STOCK_CUT)
+        item.connect('activate', self.on_cut_link)
+        m.append(item)
+        m._paste_item = item = Gtk.ImageMenuItem(Gtk.STOCK_PASTE)
+        item.connect('activate', self.on_paste)
+        m.append(item)
+        m._edit_item = item = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
+        item.connect('activate', self.on_edit_linktext)
+        m.append(item)
+        m._up_item = item = Gtk.ImageMenuItem(Gtk.STOCK_GO_UP)
+        item.connect('activate', self.on_move_link_up)
+        m.append(item)
+        m._down_item = item = Gtk.ImageMenuItem(Gtk.STOCK_GO_DOWN)
+        item.connect('activate', self.on_move_link_down)
+        m.append(item)
+        m._edit_file_item = item = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
+        item.connect('activate', self.on_edit_file)
+        m.append(item)
+        m.show_all()
+
+    def on_edit_heading(self, btn):
+        def finish_edit(text):
+            self.m_model.m_name = text
+            self.g_heading.set_markup("<b>%s</b>" % text)
+            self.g_heading.show()
+            entry.destroy()
+
+        entry = MyEntry(self.m_model.m_name, finish_edit)
+        self.heading_hbox.pack_start(entry, True, True, 0)
+        self.heading_hbox.reorder_child(entry, 0)
+        entry.show()
+
+        self.g_heading.hide()
+        entry.grab_focus()
+
+    def on_add(self, btn, event):
+        self.g_add_popup.popup(None, None, None, None, event.button, event.time)
+
     def on_remove(self, btn, event):
         self.m_parent.remove_section(self)
+
     def on_add_link_by_search(self, btn):
-        dlg = SelectLessonfileBySearchDialog()
+        dlg = SelectLessonfileBySearchDialog(editor_of(self))
         while True:
             ret = dlg.run()
             if ret == Gtk.ResponseType.OK:
@@ -298,6 +358,7 @@ class Section(Gtk.VBox):
             else:
                 break
         dlg.destroy()
+
     def on_add_link(self, btn):
         if editor_of(self).m_filename:
             open_dir = os.path.split(editor_of(self).m_filename)[0]
@@ -314,9 +375,9 @@ class Section(Gtk.VBox):
                 self._add_filenames(dlg.get_filenames())
                 break
         dlg.destroy()
+
     def _add_filenames(self, filenames):
-        for filename in filenames:
-            fn = gu.decode_filename(filename)
+        for fn in filenames:
             assert os.path.isabs(fn)
             # If the file name is a file in a subdirectory below
             # lessonfile.exercises_dir in the current working directory,
@@ -329,13 +390,16 @@ class Section(Gtk.VBox):
             except lessonfile.infocache.FileNotLessonfile:
                 continue
             self.m_model.append(fn)
-            self.g_link_box.pack_start(self.create_linkrow(fn, True, True, 0), False)
+            self.g_link_box.pack_start(self.create_linkrow(fn), True, True, 0)
+
     def on_add_link_to_new_page(self, menuitem):
         page = pd.Page(_("Untitled%s") % "", [pd.Column()])
         self.m_model.append(page)
-        self.g_link_box.pack_start(self.create_linkrow(page, True, True, 0))
+        self.g_link_box.pack_start(self.create_linkrow(page), True, True, 0)
+
     def create_linkrow(self, link_this):
         hbox = Gtk.HBox()
+
         def ff(btn, page):
             if id(page) in editor_of(self).m_page_mapping:
                 editor_of(self).show_page_id(id(page))
@@ -353,97 +417,77 @@ class Section(Gtk.VBox):
                 linkbutton = gu.ClickableLabel(lessonfile.infocache.get(link_this, 'title'))
                 linkbutton.set_tooltip_text(link_this)
             except lessonfile.InfoCache.FileNotFound:
-                linkbutton = gu.ClickableLabel(_(u"«%s» was not found") % link_this)
+                linkbutton = gu.ClickableLabel(_("«%s» was not found") % link_this)
                 linkbutton.make_warning()
 
         hbox.pack_start(linkbutton, True, True, 0)
         linkbutton.connect('button-press-event', self.on_right_click_row, link_this)
         hbox.show_all()
         return hbox
+
     def on_right_click_row(self, button, event, linked):
-        idx = self.m_model.index(linked)
+        for idx in range(len(self.m_model)):
+            if self.m_model[idx] is linked:
+                break
         if event.button == 3:
-            m = Gtk.Menu()
-            item = Gtk.ImageMenuItem(Gtk.STOCK_DELETE)
-            item.connect('activate', self.on_delete_link, linked)
-            m.append(item)
-            item = Gtk.ImageMenuItem(Gtk.STOCK_CUT)
-            item.connect('activate', self.on_cut_link, idx)
-            m.append(item)
-            item = Gtk.ImageMenuItem(Gtk.STOCK_PASTE)
-            item.set_sensitive(bool(Editor.clipboard))
-            item.connect('activate', self.on_paste, idx)
-            m.append(item)
-            item = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
-            item.connect('activate', self.on_edit_linktext, linked)
-            item.set_sensitive(bool(not isinstance(linked, basestring)))
-            m.append(item)
-            item = Gtk.ImageMenuItem(Gtk.STOCK_GO_UP)
-            item.connect('activate', self.on_move_link_up, idx)
-            item.set_sensitive(bool(idx > 0))
-            m.append(item)
-            item = Gtk.ImageMenuItem(Gtk.STOCK_GO_DOWN)
-            item.connect('activate', self.on_move_link_down, idx)
-            item.set_sensitive(bool(idx < len(self.m_model) - 1))
-            m.append(item)
-            item = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
-            item.set_sensitive(isinstance(linked, unicode))
-            item.connect('activate', self.on_edit_file, idx)
-            m.append(item)
-            m.show_all()
+            self._linked = linked
+            self._idx = idx
+            m = self.popup_menu
+            m._paste_item.set_sensitive(bool(Editor.clipboard))
+            m._edit_item.set_sensitive(bool(not isinstance(linked, str)))
+            m._up_item.set_sensitive(bool(idx > 0))
+            m._down_item.set_sensitive(bool(idx < len(self.m_model) - 1))
+            m._edit_file_item.set_sensitive(isinstance(linked, str))
             m.popup(None, None, None, None, event.button, event.time)
             return True
-    def on_delete_link(self, menuitem, linked):
-        idx = self.m_model.index(linked)
-        if id(linked) in editor_of(self).m_page_mapping:
-            editor_of(self).destroy_window(id(linked))
-        self.g_link_box.get_children()[idx].destroy()
+
+    def on_delete_link(self, menuitem):
+        if id(self._linked) in editor_of(self).m_page_mapping:
+            editor_of(self).destroy_window(id(self._linked))
+        self.g_link_box.get_children()[self._idx].destroy()
         del self.m_model[idx]
-    def on_edit_linktext(self, menuitem, linked):
-        idx = self.m_model.index(linked)
+
+    def on_edit_linktext(self, menuitem):
         # row is the hbox containing the linkbutton
-        row = self.g_link_box.get_children()[idx]
+        row = self.g_link_box.get_children()[self._idx]
         linkbutton = row.get_children()[0]
-        entry = Gtk.Entry()
-        entry.set_text(linkbutton.get_label())
+
+        def finish_edit(text):
+            linkbutton.set_label(text)
+            self.m_model[self._idx].m_name = text
+            linkbutton.show()
+            linkbutton.get_children()[0].set_alignment(0.0, 0.5)
+            entry.destroy()
+
+        entry = MyEntry(linkbutton.get_label(), finish_edit)
         row.pack_start(entry, True, True, 0)
         linkbutton.hide()
         entry.show()
         entry.grab_focus()
-        def finish_edit(entry):
-            linkbutton.set_label(entry.get_text().decode("utf-8"))
-            linkbutton.get_children()[0].set_alignment(0.0, 0.5)
-            linkbutton.show()
-            self.m_model[idx].m_name = entry.get_text().decode("utf-8")
-            entry.destroy()
-        sid = entry.connect('activate', finish_edit)
-        def keydown(entry, event):
-            if event.keyval == Gdk.KEY_Tab:
-                finish_edit(entry)
-        entry.connect('key-press-event', keydown)
-        def keyup(entry, event):
-            if event.keyval == Gdk.KEY_Escape:
-                linkbutton.show()
-                entry.disconnect(sid)
-                entry.destroy()
-                return True
-        entry.connect('key-release-event', keyup)
-    def on_edit_file(self, menuitem, linked):
+
+    def on_edit_file(self, menuitem):
         try:
             try:
                 subprocess.call((cfg.get_string("programs/text-editor"),
-                             lessonfile.uri_expand(self.m_model[linked])))
-            except OSError, e:
-                 raise osutils.BinaryForProgramException("Text editor", cfg.get_string("programs/text-editor"), e)
-        except osutils.BinaryForProgramException, e:
+                             lessonfile.uri_expand(self.m_model[self._idx])))
+            except OSError as e:
+                raise osutils.BinaryForProgramException("Text editor", cfg.get_string("programs/text-editor"), e)
+        except osutils.BinaryForProgramException as e:
             solfege.win.display_error_message2(e.msg1, e.msg2)
+
     def on_cut(self, btn):
         self.m_parent.cut_section(self)
-    def on_cut_link(self, menuitem, idx):
-        Editor.clipboard.append(self.m_model[idx])
-        del self.m_model[idx]
-        self.g_link_box.get_children()[idx].destroy()
-    def on_paste(self, btn, idx):
+
+    def on_cut_link(self, menuitem):
+        Editor.clipboard.append(self.m_model[self._idx])
+        del self.m_model[self._idx]
+        self.g_link_box.get_children()[self._idx].destroy()
+
+    def on_paste(self, btn, x):
+        if x == -1:
+            idx = -1
+        else:
+            idx = self._idx
         assert Editor.clipboard, "Paste buttons should be insensitive when the clipboard is empty."
         pobj = Editor.clipboard.pop()
         if isinstance(pobj, pd.LinkList):
@@ -452,28 +496,33 @@ class Section(Gtk.VBox):
             mobj = pobj
         if idx == -1:
             self.m_model.append(mobj)
-            self.g_link_box.pack_start(self.create_linkrow(mobj, True, True, 0))
+            self.g_link_box.pack_start(self.create_linkrow(mobj), True, True, 0)
         else:
             self.m_model.insert(idx, mobj)
             row = self.create_linkrow(mobj)
             self.g_link_box.pack_start(row, True, True, 0)
             self.g_link_box.reorder_child(row, idx)
-    def on_move_link_up(self, btn, idx):
+
+    def on_move_link_up(self, btn):
         """
         Move the link one row up.
         """
+        idx = self._idx
         assert idx > 0
         self.m_model[idx], self.m_model[idx - 1] = self.m_model[idx - 1], self.m_model[idx]
         self.g_link_box.reorder_child(self.g_link_box.get_children()[idx], idx - 1)
-    def on_move_link_down(self, btn, idx=None):
+
+    def on_move_link_down(self, btn):
         """
         Move the link one row down.
         """
+        idx = self._idx
         self.m_model[idx], self.m_model[idx + 1] = self.m_model[idx + 1], self.m_model[idx]
         self.g_link_box.reorder_child(self.g_link_box.get_children()[idx], idx + 1)
 
 
 class Column(Gtk.VBox):
+
     def __init__(self, model, parent):
         Gtk.VBox.__init__(self)
         self.set_spacing(gu.hig.SPACE_MEDIUM)
@@ -496,17 +545,21 @@ class Column(Gtk.VBox):
         b.connect('clicked', self.on_paste)
         Editor.clipboard.register_paste_button(b, pd.LinkList)
         hbox.pack_start(b, False, False, 0)
+
     def __del__(self):
         logging.debug("Column.__del__")
+
     def cut_section(self, section):
         idx = self.g_section_box.get_children().index(section)
         Editor.clipboard.append(self.m_model[idx])
         del self.m_model[idx]
         self.g_section_box.get_children()[idx].destroy()
+
     def remove_section(self, section):
         idx = self.g_section_box.get_children().index(section)
         del self.m_model[idx]
         self.g_section_box.get_children()[idx].destroy()
+
     def on_add_section(self, btn):
         # We write "Untitled%s" % "" instead of just "Untitled" here
         # since "Untitled%s" is already translated in many languages.
@@ -515,6 +568,7 @@ class Column(Gtk.VBox):
         gui_section = Section(section, self)
         self.g_section_box.pack_start(gui_section, False, False, 0)
         gui_section.show_all()
+
     def move_section_down(self, widget, section):
         idx = self.g_section_box.get_children().index(section)
         if idx < len(self.g_section_box.get_children()) - 1:
@@ -522,6 +576,7 @@ class Column(Gtk.VBox):
             self.m_model[idx], self.m_model[idx + 1] \
                     = self.m_model[idx + 1], self.m_model[idx]
             self.m_parent.update_buttons()
+
     def move_section_up(self, widget, section):
         idx = self.g_section_box.get_children().index(section)
         if idx > 0:
@@ -529,6 +584,7 @@ class Column(Gtk.VBox):
             self.m_model[idx], self.m_model[idx - 1] \
                     = self.m_model[idx - 1], self.m_model[idx]
             self.m_parent.update_buttons()
+
     def on_paste(self, widget):
         """
         Paste the clipboard as a new section to this column.
@@ -543,6 +599,7 @@ class Column(Gtk.VBox):
 
 
 class Page(Gtk.VBox):
+
     def __init__(self, model, parent):
         Gtk.VBox.__init__(self)
         self.m_model = model
@@ -558,14 +615,17 @@ class Page(Gtk.VBox):
         self.show_all()
         if model:
             self.update_from_model()
+
     def __del__(self):
         logging.debug("Page.__del__:", self.m_model.m_name)
+
     def on_add_column(self, *btn):
         column = pd.Column()
         self.m_model.append(column)
         gcol = Column(column, self)
         gcol.show_all()
         self.g_column_box.pack_start(gcol, True, True, 0)
+
     def on_move_section_left(self, button, section):
         column_idx = self.g_column_box.get_children().index(section.m_parent)
         section_idx = section.m_parent.g_section_box.get_children().index(section)
@@ -583,6 +643,7 @@ class Page(Gtk.VBox):
                 del self.m_model[-1]
                 self.g_column_box.get_children()[-1].destroy()
             self.update_buttons()
+
     def on_move_section_right(self, button, section):
         # the column we move from
         column_idx = self.g_column_box.get_children().index(section.m_parent)
@@ -593,10 +654,10 @@ class Page(Gtk.VBox):
         section.reparent(to_column.g_section_box)
         section.m_parent = to_column
         to_column.g_section_box.set_child_packing(section, False, False, 0, Gtk.PACK_START)
-        to_section_idx = len(self.m_model[column_idx + 1])
         self.m_model[column_idx + 1].append(self.m_model[column_idx][section_idx])
         del self.m_model[column_idx][section_idx]
         self.update_buttons()
+
     def update_from_model(self):
         for child in self.g_column_box.get_children():
             child.destroy()
@@ -604,13 +665,14 @@ class Page(Gtk.VBox):
             self.g_column_box.pack_start(Column(column, self), False, False, 0)
         self.g_column_box.show_all()
         self.update_buttons()
+
     def update_buttons(self):
         num_cols = len(self.g_column_box.get_children())
         for col_idx, column in enumerate(self.g_column_box.get_children()):
             num_sects = len(column.g_section_box.get_children())
             for sect_idx, section in enumerate(column.g_section_box.get_children()):
                 section.g_move_up_btn.set_sensitive(sect_idx != 0)
-                section.g_move_down_btn.set_sensitive(sect_idx != num_sects -1)
+                section.g_move_down_btn.set_sensitive(sect_idx != num_sects - 1)
                 section.g_move_left_btn.set_sensitive(col_idx != 0)
                 if [col for col in self.g_column_box.get_children() if not col.g_section_box.get_children()] and col_idx == num_cols - 1:
                     section.g_move_right_btn.set_sensitive(False)
@@ -619,28 +681,34 @@ class Page(Gtk.VBox):
 
 
 class Clipboard(list):
+
     def __init__(self, v=[]):
         list.__init__(v)
         self.m_paste_buttons = []
+
     def pop(self, i=-1):
         ret = list.pop(self, i)
         self.update_buttons()
         return ret
+
     def append(self, obj):
         list.append(self, obj)
         self.update_buttons()
+
     def register_paste_button(self, button, accepts_types):
         button.set_sensitive(bool(self) and isinstance(self[-1], accepts_types))
         self.m_paste_buttons.append((button, accepts_types))
+
     def update_buttons(self):
         for button, types in self.m_paste_buttons:
             button.set_sensitive(bool(self) and isinstance(self[-1], types))
 
 
 class Editor(Gtk.Window, gu.EditorDialogBase):
-    savedir = os.path.join(filesystem.user_data(), u'exercises', u'user')
+    savedir = os.path.join(filesystem.user_data(), 'exercises', 'user')
     # The clipboard will be shared between all Editor instances
     clipboard = Clipboard()
+
     def __init__(self, filename=None):
         Gtk.Window.__init__(self)
         logging.debug("fpeditor.Editor.__init__(%s)", filename)
@@ -656,7 +724,7 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
         self.g_title_hbox.set_spacing(gu.hig.SPACE_SMALL)
         self.g_title_hbox.set_border_width(gu.hig.SPACE_SMALL)
         label = Gtk.Label()
-        label.set_markup(u"<b>%s</b>" % _("Front page title:"))
+        label.set_markup("<b>%s</b>" % _("Front page title:"))
         self.g_title_hbox.pack_start(label, False, False, 0)
         self.g_fptitle = Gtk.Entry()
         self.g_title_hbox.pack_start(self.g_fptitle, True, True, 0)
@@ -677,8 +745,10 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
         self.add_to_instance_dict()
         self.g_fptitle.set_text(self.m_model.m_name)
         self.g_fptitle.connect('changed', self.on_frontpage_title_changed)
+
     def __del__(self):
         logging.debug("fpeditor.Editor.__del__, filename=%s", self.m_filename)
+
     def add_page(self, page):
         """
         Add and show the page.
@@ -686,8 +756,10 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
         editor_of(self).m_page_mapping[id(page.m_model)] = page
         self.g_main_box.pack_start(page, True, True, 0)
         self.show_page(page)
+
     def show_page_id(self, page_id):
         self.show_page(self.m_page_mapping[page_id])
+
     def show_page(self, page):
         """
         Hide the currently visible page, and show PAGE instead.
@@ -704,10 +776,13 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
             self.g_title_hbox.show()
         self.g_ui_manager.get_widget("/Toolbar/GoBack").set_sensitive(
             not isinstance(self.g_visible_page.m_parent, Editor))
+
     def go_back(self, *action):
         self.show_page(self.g_visible_page.m_parent)
+
     def on_frontpage_title_changed(self, widget):
         self.m_model.m_name = widget.get_text()
+
     def setup_toolbar(self):
         self.g_ui_manager.insert_action_group(self.g_actiongroup, 0)
         uixml = """
@@ -732,6 +807,7 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
         self.g_main_box.pack_start(toolbar, False, False, 0)
         self.g_main_box.reorder_child(toolbar, 0)
         self.g_ui_manager.get_widget("/Toolbar").set_style(Gtk.ToolbarStyle.BOTH)
+
     def destroy_window(self, window_id):
         """
         Destroy the window with the id 'windowid' and all subwindows.
@@ -744,6 +820,7 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
             editor_of(self).m_page_mapping[wid].destroy()
             del editor_of(self).m_page_mapping[wid]
         do_del(window_id)
+
     @staticmethod
     def edit_file(fn):
         if fn in Editor.instance_dict:
@@ -752,14 +829,15 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
             try:
                 win = Editor(fn)
                 win.show()
-            except IOError, e:
+            except IOError as e:
                 gu.dialog_ok(_("Loading file '%(filename)s' failed: %(msg)s") %
-                        {'filename': fn, 'msg': str(e).decode('utf8', 'replace')})
+                        {'filename': fn, 'msg': str(e)}, self)
+
     def load_file(self, filename):
         """
         Load a file into a empty, newly created Editor object.
         """
-        assert self.m_model == None
+        assert self.m_model is None
         self.m_model = pd.load_tree(filename, C_locale=True)
         self.m_filename = filename
         #
@@ -771,37 +849,42 @@ class Editor(Gtk.Window, gu.EditorDialogBase):
                 m.destroy()
         self.set_not_modified()
         self.set_title(self.m_filename)
+
     def set_not_modified(self):
         """
         Store the current state of the data in self.m_orig_dump so that
         is_modified() will return False until we make new changes.
         """
-        io = StringIO.StringIO()
-        self.m_model.dump(io)
-        self.m_orig_dump = io.getvalue()
+        s = io.StringIO()
+        self.m_model.dump(s)
+        self.m_orig_dump = s.getvalue()
+
     def is_modified(self):
         """
         Return True if the data has changed since the last call to
         set_not_modified()
         """
-        io = StringIO.StringIO()
-        self.m_model.dump(io)
-        s = io.getvalue()
+        s = io.StringIO()
+        self.m_model.dump(s)
+        s = s.getvalue()
         return s != self.m_orig_dump
+
     @property
     def m_changed(self):
         return self.is_modified()
+
     def save(self, w=None):
         assert self.m_filename
-        save_location = os.path.split(self.m_filename)[0] + os.sep
         fh = pd.FileHeader(1, self.m_model)
         fh.save_file(self.m_filename)
         self.set_not_modified()
         # We do test for solfege.win since it is not available during testing
         if hasattr(solfege, 'win'):
             solfege.win.load_frontpage()
+
     def on_show_help(self, *w):
         return
+
     def get_save_as_dialog(self):
         dialog = gu.EditorDialogBase.get_save_as_dialog(self)
         ev2 = Gtk.EventBox()
