@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011, 2016  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from __future__ import absolute_import
-
 import os
 import re
 import signal
@@ -37,7 +35,7 @@ def get_drives():
     """
     # First we do this with maxlen = 1 to figure out how long string we need.
     i = ctypes.c_int(0)
-    v = ctypes.create_string_buffer('\000')
+    v = ctypes.create_string_buffer(b'\000')
     retval = ctypes.windll.kernel32.GetLogicalDriveStringsW(i, v)
     if retval == 0:
         raise Exception("get_drives failed. Retval==0")
@@ -45,56 +43,62 @@ def get_drives():
     # So now we get the real data.
     maxlen = retval
     i = ctypes.c_int(maxlen - 1)
-    v = ctypes.create_string_buffer('\000' * maxlen)
+    v = ctypes.create_string_buffer(b'\000' * maxlen)
     retval = ctypes.windll.kernel32.GetLogicalDriveStringsA(i, v)
-    return [x for x in v.raw.split("\000") if x]
+    return [str(x) for x in bytes(v).split(b"\x00") if x]
 
 
 class OsUtilsException(Exception):
     pass
 
+
 class RunningExecutableFailed(OsUtilsException):
+
     def __init__(self, cmd):
         OsUtilsException.__init__(self,
           _("Running the command %s failed.") % cmd)
 
+
 class ExecutableDoesNotExist(OsUtilsException):
     """
     We should raise this exception if os.system fails and returns
     anything else than 0.
     """
+
     def __init__(self, cmd):
         OsUtilsException.__init__(self,
           _("The executable '%s' does not exist.") % cmd)
 
-class ExecutableFormatStringException(OsUtilsException):
-    def __init__(self):
-        OsUtilsException.__init__(self,
-          _("The executable string should contain zero or one '%s'."))
 
 class BinaryBaseException(OsUtilsException):
+
     def __init__(self, binary, exception):
         OsUtilsException.__init__(self)
         self.msg2 = _("Tried `%(bin)s`:\n\n\t%(exception)s\n\n"
             "Please check that the program is installed. If you did not supply "
-            "the full path to the program in the preferences window (Ctrl-12), "
+            "the full path to the program in the preferences window (Ctrl-F12), "
             "you must make sure the program is on your PATH.") % {
-             'bin': str(binary).decode(sys.getfilesystemencoding(), 'replace'),
-             'exception': str(exception).decode(
-                 sys.getfilesystemencoding(), 'replace')}
+             'bin': binary,
+             'exception': str(exception)}
+
 
 class BinaryForProgramException(BinaryBaseException):
+
     def __init__(self, program_name, binary, exception):
         BinaryBaseException.__init__(self, binary, exception)
         self.msg1 = _("Failed to execute a binary for %s") % program_name
 
+
 class BinaryForMediaPlayerException(BinaryBaseException):
+
     def __init__(self, typeid, binary, exception):
         BinaryBaseException.__init__(self, binary, exception)
         self.msg1 = _("Failed to execute media player for %s files") % typeid.upper()
 
+
 class BinaryForMediaConvertorException(BinaryBaseException):
     r = re.compile("app/(?P<from>[a-z0-9]+)_to_(?P<to>[a-z0-9]+)_cmd")
+
     def __init__(self, varname, binary, exception):
         BinaryBaseException.__init__(self, binary, exception)
         m = self.r.match(varname)
@@ -107,7 +111,6 @@ class BinaryForMediaConvertorException(BinaryBaseException):
 __all__ = [
     'OsUtilsException',
     'ExecutableDoesNotExist',
-    'ExecutableFormatStringException',
     'BinaryForProgramException',
 ]
 
@@ -119,8 +122,10 @@ if sys.version_info < (2, 6):
         We define this class because Popen.kill was added in Python 2.6,
         and we want to run with 2.5 too.
         """
+
         def __init__(self, *args, **kwargs):
             subprocess.Popen.__init__(self, *args, **kwargs)
+
         def kill(self):
             if sys.platform == 'win32':
                 # http://code.activestate.com/recipes/347462/
@@ -134,6 +139,7 @@ if sys.version_info < (2, 6):
 else:
     Popen = subprocess.Popen
 
+
 class PopenSingleton(object):
     """
     A special version of Popen that will kill the previous program
@@ -182,7 +188,7 @@ def find_progs(execs):
 
     execs - a tuple of possible executable names
     """
-    assert isinstance(execs, (list, tuple,))
+    assert isinstance(execs, (list, tuple))
     retval = set()
     for p in os.environ['PATH'].split(os.pathsep):
         for e in execs:
@@ -208,7 +214,6 @@ def find_mma_executables(ignore_drives):
             try:
                 for dirname in os.listdir(drive):
                     if dirname.lower().startswith('mma'):
-                        dir = os.path.join(drive, dirname)
                         mma_py = os.path.join(drive, dirname, "mma.py")
                         if os.path.exists(mma_py):
                             retval.add(mma_py)
@@ -219,10 +224,11 @@ def find_mma_executables(ignore_drives):
                 # People have had this exception raised:
                 # WindowsError: [Error 21] The device is not ready: 'X:\\*.*'
                 # So we choose to continue with the next drive if anything
-                # unexcepted happens. 
+                # unexcepted happens.
                 continue
     return retval
 
+
 def find_csound_executables():
     """
     Return a list of possible csound executables.
@@ -236,4 +242,3 @@ def find_csound_executables():
         if os.path.exists(fn):
             retval.add(fn)
     return retval
-
